#! -*- coding: utf-8 -*-
import locale
import logging
import datetime
import xml.etree.ElementTree as ET
import socket

from decimal import Decimal, ROUND_UP
from suds.client import Client
from suds.xsd.doctor import Import, ImportDoctor

from pysimplesoap.client import SoapClient

from django.conf import settings

from acomea.soap.utils import format_data, get_active_button_dict
from acomea.soap import fakedata

__author__ = 'simone cittadini'

log = logging.getLogger("acomea.soap")

SOAP_FAKEDATA = False

SOAP_PUBLIC_URL = getattr(settings, 'ACOMEA_SOAP_PUBLIC_URL', 'http://93.57.3.196:8284/service.asmx?wsdl')
SOAP_PUBLIC_SERVICE = getattr(settings, 'ACOMEA_SOAP_PUBLIC_SERVICE', 'http://tempuri.org/AcomeA.SoapAP/Service')
SOAP_RESERVED_URL = getattr(settings, 'ACOMEA_SOAP_RESERVED_URL', 'http://93.57.3.196:8282/service.asmx?wsdl')
SOAP_RESERVED_SERVICE = getattr(settings, 'SOAP_RESERVED_SERVICE', 'http://tempuri.org/AcomeA.SoapAR/Service')

SOAP_G5_URL = 'http://93.57.3.196:8286/service.asmx?wsdl'


class AcomeaClient:
    """
    Questo client è più performante, ma ha problemi a girare in alcuni ambienti, tipo il cms
    """

    def __init__(self):
        log.debug("AcomeaClient init")
        imp = Import('http://www.w3.org/2001/XMLSchema')
        imp.filter.add('http://tempuri.org/AcomeA.SoapAP/Service')
        doctor = ImportDoctor(imp)
        try:
            self.client = Client(SOAP_PUBLIC_URL, doctor=doctor)
        except Exception, exc:
            log.warning("AcomeaClient Exception %s" % str(exc))
            raise

    def get_fondi_per_tipo(self, cod_tipo):
        """
        Torna una struttura dati con le info principali di tutti i fondi per un certo tipo.
        """
        return self.client.service.GetFondiDSxTipo(cod_tipo).diffgram.NewDataSet.Table

    def get_last_data_quote(self):
        """
        Ritorna l'ultima data per qui esistono dei valori delle quote.
        """
        d = self.client.service.DataGiro()
        return datetime.date(d.year, d.month, d.day)

    def get_valori_riga_tabella_quote(self, cod_fondo, classe, data=datetime.date.today()):
        """
        Torna i valori per la tabella quote relativi ad un fondo (passato come stringa codice fondo).
        """
        strdata = str(data)[0:10]
        return self.client.service.CreateCFondoComuneByCodeAndDatexClasse(cod_fon=cod_fondo, dt=strdata, cq=classe)

    def check_cliente(self, cod_fiscale):
        """
        Torna le informazioni di esistenza / permessi per un cliente dato il suo codice fiscale.
        """
        return self.client.service.CheckCliente(cf=cod_fiscale)

    def get_cliente(self, cod_fiscale):
        """
        Torna una struttura dati con le informazioni relative ad un cliente dato il suo codice fiscale.
        """
        data = self.client.service.GetCliente(cf=cod_fiscale).diffgram.NewDataSet.Table
        # quando il cliente ha una sola gestione e non entrambe il soap ritorna un'instanza
        if type(data) != type(list()):
            data = [data]
        return data


class AcomeaClientCms:
    """
    Client più lento, ma che non da problemi usato da cms
    """

    def __init__(self):
        self._client = None

    @property
    def client(self):
        if not self._client:
            if SOAP_FAKEDATA:
                self._client = SoapClient()
                return
            try:
                self._client = SoapClient(wsdl=SOAP_PUBLIC_URL, trace=False)
            except socket.timeout as err:
                log.warning("AcomeaClientCms init timeout: %s" % str(err))
                self._client = None
            except Exception, exc:
                log.warning("AcomeaClientCms Exception %s" % str(exc))
                raise
        return self._client

    def get_fondi_per_tipo(self, cod_tipo):
        """
        Torna una struttura dati con le info principali di tutti i fondi per un certo tipo.
        """
        return self.client.GetFondiDSxTipo(cod_tipo).diffgram.NewDataSet.Table

    def get_last_data_quote(self):
        """
        Ritorna l'ultima data per qui esistono dei valori delle quote.
        """
        d = self.client.DataGiro()['DataGiroResult']
        return datetime.date(d.year, d.month, d.day)

    def get_valori_riga_tabella_quote(self, cod_fondo, classe, data):
        """
        Torna i valori per la tabella quote relativi ad un fondo (passato come stringa codice fondo).
        """
        g, m, a = str(data).split('/')
        if len(m) == 1:
            m = "0%s" % m
        if len(g) == 1:
            g = "0%s" % g
        strdata = "%s-%s-%s" % (a, m, g)
        data = self.client.CreateCFondoComuneByCodeAndDatexClasse(cod_fon=cod_fondo, dt=strdata, cq=classe)
        return data['NewCFonByCodeAndDatexClasseResult']

    def check_cliente(self, cod_fiscale):
        """
        Torna le informazioni di esistenza / permessi per un cliente dato il suo codice fiscale.
        """
        return self.client.CheckCliente(cf=cod_fiscale)['CheckClienteResult']

    def get_cliente(self, cod_fiscale):
        """
        Torna una struttura dati con le informazioni relative ad un cliente dato il suo codice fiscale.
        """
        data = self.client.GetCliente(cf=cod_fiscale)['GetClienteResult']
        root = ET.fromstring(data.as_xml(pretty=True))
        output = []
        for child in root.iter('Table'):
            dict = {}
            for element in child:
                dict[element.tag] = element.text
            output.append(dict)
        return output

    def get_fondi(self):
        data = self.client.GetFondiDS()['GetFondiDSResult']
        #todo, è un <class 'pysimplesoap.simplexml.SimpleXMLElement'>, farlo diventare dict
        return data

    def get_classi_di_attivita(self, cod_fondo):
        if SOAP_FAKEDATA:
            return fakedata.get_classi_di_attivita()
        ff = self.client.CreateFondiAnalisi(Fondo=cod_fondo, idLingua="IT")['CreateFondiAnalisiResult']

        if ff['HasComposizionePatrimoniale']:
            data = ff['ComposizionePatrimoniale']
            xmldata = data['DS']
            categories = []
            series_data = []
            pie_data = []
            root = ET.fromstring(xmldata.as_xml(pretty=True))
            for child in root.iter('ComposizionePatrimoniale'):
                pie_entry = []
                for c in list(child):
                    if c.tag == 'Voce':
                        categories.append(c.text)
                        pie_entry.append(c.text)
                    if c.tag == 'Val1':
                        series_data.append(float(c.text))
                        pie_entry.append(float(c.text))
                pie_data.append(pie_entry)
            # i dati che arrivano dal soap sono ripetuti due volte
            series_data = series_data[0:len(series_data) / 2]
            pie_data = pie_data[0:len(pie_data) / 2]

            output = dict()
            output['line'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': series_data
            }
            output['pie'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': pie_data
            }
            output['active_buttons'] = get_active_button_dict(ff)
            return output

        return None

    def get_titoli(self, cod_fondo):
        if SOAP_FAKEDATA:
            return fakedata.get_titoli()
        ff = self.client.CreateFondiAnalisi(Fondo=cod_fondo, idLingua='IT')['CreateFondiAnalisiResult']

        if ff['HasPrincipaliTitoli']:
            data = ff['PrincipaliTitoliPortafoglio']
            xmldata = data['DS']
            categories = []
            series_data = []
            pie_data = []
            root = ET.fromstring(xmldata.as_xml(pretty=True))
            for child in root.iter('Titoli'):
                pie_entry = []
                for c in list(child):
                    if c.tag == 'Voce':
                        categories.append(c.text)
                        pie_entry.append(c.text)
                    if c.tag == 'Val1':
                        series_data.append(float(c.text))
                        pie_entry.append(float(c.text))
                pie_data.append(pie_entry)
            # i dati che arrivano dal soap sono ripetuti due volte
            series_data = series_data[0:len(series_data) / 2]
            pie_data = pie_data[0:len(pie_data) / 2]
            output = dict()
            output['line'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': series_data
            }

            output['pie'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': pie_data
            }
            output['active_buttons'] = get_active_button_dict(ff)
            return output

        return None

    def get_rating(self, cod_fondo):
        if SOAP_FAKEDATA:
            return fakedata.get_rating()
        ff = self.client.CreateFondiAnalisi(Fondo=cod_fondo, idLingua='IT')['CreateFondiAnalisiResult']

        if ff['HasRatings']:
            data = ff['Ratings']
            xmldata = data['DS']
            categories = []
            series_data = []
            pie_data = []
            root = ET.fromstring(xmldata.as_xml(pretty=True))
            for child in root.iter('Ratings'):
                pie_entry = []
                for c in list(child):
                    if c.tag == 'Voce':
                        categories.append(c.text)
                        pie_entry.append(c.text)
                    if c.tag == 'Val1':
                        series_data.append(float(c.text))
                        pie_entry.append(float(c.text))
                pie_data.append(pie_entry)
            # i dati che arrivano dal soap sono ripetuti due volte
            series_data = series_data[0:len(series_data) / 2]
            pie_data = pie_data[0:len(pie_data) / 2]

            output = dict()
            output['line'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': series_data
            }
            output['pie'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': pie_data
            }
            output['active_buttons'] = get_active_button_dict(ff)
            return output

        return None

    def get_valute(self, cod_fondo):
        if SOAP_FAKEDATA:
            return fakedata.get_valute()
        ff = self.client.CreateFondiAnalisi(Fondo=cod_fondo, idLingua='IT')['CreateFondiAnalisiResult']

        if ff['HasRPV']:
            data = ff['RipartizionePrincipaliValute']
            xmldata = data['DS']
            categories = []
            series_data = []
            pie_data = []
            root = ET.fromstring(xmldata.as_xml(pretty=True))
            for child in root.iter('Valute'):
                pie_entry = []
                for c in list(child):
                    if c.tag == 'Voce':
                        categories.append(c.text)
                        pie_entry.append(c.text)
                    if c.tag == 'Val1':
                        series_data.append(float(c.text))
                        pie_entry.append(float(c.text))
                pie_data.append(pie_entry)
            # i dati che arrivano dal soap sono ripetuti due volte
            series_data = series_data[0:len(series_data) / 2]
            pie_data = pie_data[0:len(pie_data) / 2]

            output = dict()
            output['line'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': series_data
            }
            output['pie'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': pie_data
            }
            output['active_buttons'] = get_active_button_dict(ff)
            return output

        return None

    def get_principali_paesi(self, cod_fondo):
        if SOAP_FAKEDATA:
            return fakedata.get_principali_paesi()
        ff = self.client.CreateFondiAnalisi(Fondo=cod_fondo, idLingua='IT')['CreateFondiAnalisiResult']
        if ff['HasPrincipaliPaesi']:
            data = ff['PrincipaliPaesiInvestimento']
            xmldata = data['DS']
            categories = []
            series_data = []
            pie_data = []
            root = ET.fromstring(xmldata.as_xml(pretty=True))
            for child in root.iter('Paesi'):
                pie_entry = []
                for c in list(child):
                    if c.tag == 'Voce':
                        categories.append(c.text)
                        pie_entry.append(c.text)
                    if c.tag == 'Val1':
                        series_data.append(float(c.text))
                        pie_entry.append(float(c.text))
                pie_data.append(pie_entry)
            # i dati che arrivano dal soap sono ripetuti due volte
            series_data = series_data[0:len(series_data) / 2]
            pie_data = pie_data[0:len(pie_data) / 2]

            output = dict()
            output['line'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': series_data
            }
            output['pie'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': pie_data
            }
            output['active_buttons'] = get_active_button_dict(ff)
            return output

        return None

    def get_scadenza(self, cod_fondo):
        if SOAP_FAKEDATA:
            return fakedata.get_scadenza()
        ff = self.client.CreateFondiAnalisi(Fondo=cod_fondo, idLingua='IT')['CreateFondiAnalisiResult']

        if ff['HasScadenza']:
            data = ff['Scadenza']
            xmldata = data['DS']
            categories = []
            series_data = []
            pie_data = []
            root = ET.fromstring(xmldata.as_xml(pretty=True))
            for child in root.iter('Scadenza'):
                pie_entry = []
                for c in list(child):
                    if c.tag == 'Voce':
                        categories.append(c.text)
                        pie_entry.append(c.text)
                    if c.tag == 'Val1':
                        series_data.append(float(c.text))
                        pie_entry.append(float(c.text))
                pie_data.append(pie_entry)
            # i dati che arrivano dal soap sono ripetuti due volte
            series_data = series_data[0:len(series_data) / 2]
            pie_data = pie_data[0:len(pie_data) / 2]

            output = dict()
            output['line'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': series_data
            }
            output['pie'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': pie_data
            }
            output['active_buttons'] = get_active_button_dict(ff)
            return output

        return None

    def get_aree_geografiche(self, cod_fondo):
        if SOAP_FAKEDATA:
            return fakedata.get_aree_geografiche()
        ff = self.client.CreateFondiAnalisi(Fondo=cod_fondo, idLingua='IT')['CreateFondiAnalisiResult']

        if ff['HasAreeGeografiche']:
            data = ff['AreeGeografiche']
            xmldata = data['DS']
            categories = []
            graph_categories = []
            series_data = []
            graph_series_data = []
            pie_data = []
            graph_pie_data = []
            root = ET.fromstring(xmldata.as_xml(pretty=True))
            # import ipdb; ipdb.set_trace()
            for child in root.iter('AreeGeografiche'):
                pie_entry = []
                graph_pie_entry = []
                bold = True if getattr(child.find('Bold'), 'text', None) == '1' else False
                for ch in list(child):
                    if ch.tag == 'Voce':
                        categories.append(ch.text)
                        pie_entry.append(ch.text)
                        if bold:
                            graph_categories.append(ch.text)
                        else:
                            graph_pie_entry.append(ch.text)
                    if ch.tag == 'Val1':
                        series_data.append(float(ch.text))
                        pie_entry.append(float(ch.text))
                        if bold:
                            graph_series_data.append(float(ch.text))
                        else:
                            graph_pie_entry.append(float(ch.text))
                pie_data.append(pie_entry)
                graph_pie_data.append(graph_pie_entry)
            # i dati che arrivano dal soap sono ripetuti due volte
            series_data = series_data[0:len(series_data) / 2]
            pie_data = pie_data[0:len(pie_data) / 2]
            graph_series_data = graph_series_data[0:len(graph_series_data) / 2]
            graph_pie_data = graph_pie_data[0:len(graph_pie_data) / 2]
            graph_pie_data = [l for l in graph_pie_data if l != []]
            graph_pie_data = [l for l in graph_pie_data if l != []]
            graph_pie_data = [[x[0].replace('---', ''), x[1]] for x in graph_pie_data if isinstance(x[0], str)]
            output = dict()
            output['line'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': graph_categories,
                'SeriesData': graph_series_data
            }
            output['table'] = pie_data
            output['pie'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': graph_categories,
                'SeriesData': graph_pie_data
            }
            output['active_buttons'] = get_active_button_dict(ff)
            output['HasAreeGeografiche'] = True
            return output
        return None

    def get_andamento(self, cod_fondo, classe):
        if SOAP_FAKEDATA:
            return fakedata.get_andamento()

        if not self.client:
            return

        ff = self.client.CreateCFondoComuneByCodexClasse(cod_fon=cod_fondo, cq=classe)['NewCFonByCodexClasseResult']
        return ff

    def get_serie_storica(self, cod_fondo, classe, data_riferimento=None):
        if SOAP_FAKEDATA:
            return fakedata.get_serie_storica()
        if data_riferimento:
            ff = self.client.GetSerieStoricaByDate(cod_fon=cod_fondo, dt=data_riferimento)
            xmldata = ff['GetSerieStoricaByDateResult']
        else:
            ff = self.client.GetSerieStorica(cod_fon=cod_fondo)
            xmldata = ff['GetSerieStoricaResult']

        root = ET.fromstring(xmldata.as_xml(pretty=True))
        output = []
        for child in root.iter('Table'):
            linea = {}
            for c in list(child):
                if c.tag == 'Data':
                    linea['data'] = c.text
                if c.tag == 'QuotaA1':
                    linea['quota_a1'] = c.text
                if c.tag == 'QuotaA2':
                    linea['quota_a2'] = c.text
                if c.tag == 'Quota':
                    linea['quota'] = c.text
            output.append(linea)
        return output

    def get_serie_storica_per_classe(self, cod_fondo, classe, data_riferimento=None):
        if SOAP_FAKEDATA:
            return fakedata.get_serie_storica()
        if data_riferimento:
            ff = self.client.GetSerieStoricaByDatexClasse(cod_fon=cod_fondo, dt=data_riferimento, classe=classe)
            xmldata = ff['GetSerieStoricaByDatexClasseResult']
        else:
            ff = self.client.GetSerieStoricaxClasse(cod_fon=cod_fondo, classe=classe)
            xmldata = ff['GetSerieStoricaxClasseResult']

        output = [
            dict(
                data=row['SerieStoricaItems']['Data'].strftime("%Y-%m-%d"),
                quota=row['SerieStoricaItems']['Quota'],
                quota_rettificata=row['SerieStoricaItems']['QuotaRettificata'],
                # fondo=row['SerieStoricaItems']['Fondo']
            ) for row in xmldata
        ]
        return output

    def get_rendimenti_annui(self, cod_fondo, classe):
        if SOAP_FAKEDATA:
            return fakedata.get_rendimenti_annui()
        ff = self.client.CreateFondiRendimentiXFondoxClasse(cod_fon=cod_fondo, cq=classe)
        data = ff['RendimentiXFondoxClasseResult']
        xmldata = data['DS']
        data_aggiornamento = data['DTGIRO']
        root = ET.fromstring(xmldata.as_xml(pretty=True))
        output = []
        for child in root.iter('Serie'):
            linea = {}
            for c in list(child):
                if c.tag == 'CodFon':
                    linea['codfon'] = c.text
                if c.tag == 'Fondo':
                    linea['fondo'] = c.text
                if c.tag == 'Anno':
                    linea['anno'] = c.text
                if c.tag == 'ValF':
                    linea['valf'] = c.text
                if c.tag == 'ValB':
                    linea['valb'] = c.text
                linea['data_aggiornamento'] = data_aggiornamento
            output.append(linea)
        return output

    def get_stacchi_cedola(self, cod_fondo, classe):
        if SOAP_FAKEDATA:
            return fakedata.get_stacchi_cedola()
        ff = self.client.GetStacchiCedola(cod_fon=cod_fondo, classe=classe)
        xmldata = ff['GetStacchiCedolaResult']
        # output = [
        #     dict(
        #         data=row['SerieStoricaItems']['Data'].strftime("%Y-%m-%d"),
        #         quota=row['SerieStoricaItems']['Quota'],
        #         # fondo=row['SerieStoricaItems']['Fondo']
        #     ) for row in xmldata
        # ]

        root = ET.fromstring(xmldata.as_xml(pretty=True))
        output = []
        for child in root.iter('Table'):
            linea = {}
            for c in list(child):
                if c.tag == 'DATA':
                    linea['data'] = c.text
                if c.tag == 'PERC':
                    linea['perc'] = c.text
            output.append(linea)
        return output

    def get_grafico(self, cod_fondo, classe, data_dal=None, data_al=None):
        if SOAP_FAKEDATA:
            return fakedata.get_grafico()
        if data_dal and data_al:
            ff = self.client.CreateFondiAndamentoByCodeAndDatexClasse(
                cod_fon=cod_fondo, classe=classe,
                dt1=data_dal.strftime("%Y-%m-%d"), dt2=data_al.strftime("%Y-%m-%d")
            )
            data = ff['FondiAndamento']
        else:
            ff = self.client.CreateFondiAndamentoByCodexClasse(cod_fon=cod_fondo, classe=classe)
            data = ff['FondiAndamento']
        xmldata = data['DS']
        root = ET.fromstring(xmldata.as_xml(pretty=True))
        series = []
        for child in root.iter('NewDataSet'):
            for c in list(child):

                linea = {}
                for e in list(c):
                    if e.tag == 'Data':
                        linea['data'] = e.text
                    if e.tag == 'ValF':
                        linea['valf'] = round(float(e.text), 3)
                    if e.tag == 'ValB':
                        linea['valb'] = round(float(e.text), 3)
                if 'data' in linea:
                    series.append(linea)

        output = {
            'PerfF': data['PerfF'] if 'PerfF' in data else None,
            'HasBenchmark': data['HasBenchmark'] if 'HasBenchmark' in data else None,
            'PerfB': data['PerfB'] if 'PerfB' in data else None,
            'BenchCode': data['BenchCode'] if 'BenchCode' in data else None,
            'DataEffettiva1': data['DataEffettiva1'] if 'DataEffettiva1' in data else None,
            'DataEffettiva2': data['DataEffettiva2'] if 'DataEffettiva2' in data else None,
            'Legenda': data['Legenda'] if 'Legenda' in data else None,
            'StartValB': data['StartValB'] if 'StartValB' else None,
            'StartValF': data['StartValF'] if 'StartValF' else None,
            'BenchDescr': data['BenchDescr'] if 'BenchDescr' in data else None,
            'SeriesCount': data['SeriesCount'] if 'SeriesCount' in data else None,
            'TipoFondo': data['TipoFondo'] if 'TipoFondo' in data else None,
            'Sottotitolo': data['Sottotitolo'] if 'Sottotitolo' in data else None,
            'TFRDataCount': data['TFRDataCount'] if 'TFRDataCount' in data else None,
            'DescrFon': data['DescrFon'] if 'DescrFon' in data else None,
            'MaxVal': data['MaxVal'] if 'MaxVal' in data else None,
            'DTINIZIOVALQUOTA': data['DTINIZIOVALQUOTA'] if 'DTINIZIOVALQUOTA' in data else None,
            'LegendaTFR': data['LegendaTFR'] if 'LegendaTFR' in data else None,
            'MinVal': data['MinVal'] if 'MinVal' in data else None,
            'CLASSE': data['CLASSE'] if 'CLASSE' in data else None,
            'DTGIRO': data['DTGIRO'] if 'DTGIRO' in data else None,
            'EndValB': data['EndValB'] if 'EndValB' in data else None,
            'EndValF': data['EndValF'] if 'EndValF' in data else None,
            'SeriesData': series
        }
        return output

    def get_settori(self, cod_fondo):
        if SOAP_FAKEDATA:
            return fakedata.settori()
        ff = self.client.CreateFondiAnalisi(Fondo=cod_fondo, idLingua='IT')['CreateFondiAnalisiResult']

        if ff['HasSettori']:
            data = ff['Settori']
            xmldata = data['DS']
            categories = []
            series_data = []
            pie_data = []
            root = ET.fromstring(xmldata.as_xml(pretty=True))
            for child in root.iter('Settori'):
                pie_entry = []
                for c in list(child):
                    if c.tag == 'Voce':
                        categories.append(c.text)
                        pie_entry.append(c.text)
                    if c.tag == 'Val1':
                        series_data.append(float(c.text))
                        pie_entry.append(float(c.text))
                pie_data.append(pie_entry)
            # i dati che arrivano dal soap sono ripetuti due volte
            series_data = series_data[0:len(series_data) / 2]
            pie_data = pie_data[0:len(pie_data) / 2]
            output = dict()
            series = [{
                'name': 'fondo',
                'data': series_data
            }]
            output['line'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': series_data
            }

            output['pie'] = {
                'DataAggiornamento': data['DataAggiornamento'],
                'Titolo': data['Titolo'],
                'IDTAB': data['IDTAB'],
                'SottoTitolo': data['SottoTitolo'],
                'Precalcolata': data['Precalcolata'],
                'Righe': data['Righe'],
                'Colonne': data['Colonne'],
                'Ordine': data['Ordine'],
                'ID': data['ID'],
                'TitoloBreve': data['TitoloBreve'],
                'Categories': categories,
                'SeriesData': pie_data
            }
            output['active_buttons'] = get_active_button_dict(ff)
            return output

        return None


class AcomeaClientRiservata:
    def __init__(self):
        log.debug("AcomeaClientRiservata init")
        imp = Import('http://www.w3.org/2001/XMLSchema')
        imp.filter.add(SOAP_RESERVED_SERVICE)
        doctor = ImportDoctor(imp)
        try:
            self.client = Client(SOAP_RESERVED_URL, doctor=doctor)
        except Exception, exc:
            log.warning("AcomeaClientRiservata Exception %s" % str(exc))
            raise


# ecommerce
class AcomeaClientRiservataCms:
    """
    Client più lento, ma che non da problemi usato da cms
    """

    def __init__(self):
        log.debug("AcomeaClientRiservataCms init")
        try:
            self.client = SoapClient(wsdl=SOAP_RESERVED_URL, trace=False)
        except Exception, exc:
            log.warning("AcomeaClientRiservataCms Exception %s" % str(exc))
            raise

    def is_area_active(self):
        """
        http://93.57.3.196:8282/service.asmx?op=AreaRiservata_IsActive
        """
        data = self.client.AreaRiservata_IsActive()
        return data['AreaRiservata_IsActiveResult']

    def get_cliente(self, cod_fiscale):
        """
        Torna una struttura coi dati principali di un cliente dato il suo codice fiscale,
        o None se il cliente non esiste.
        """
        data = self.client.GetCliente(cod_fiscale)['GetClienteResult']
        if data['CODFISC']:
            return data
        else:
            return None

    def op_online_datagrid(self, cod_cli, tipo_partecipazione):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_DataGrid
        <CodCliente>int</CodCliente>
        <tp>INTESTATARIO or COINTESTATARIO or DELEGATO</tp>
        quali tipi di partecipazioni devo visualizzare dipendono da get_cliente
        """
        # fare ciclo for
        return self.client.OpOnline_DataGrid(cod_cli, tipo_partecipazione)['OpOnline_DataGridResult']

    def qst_must_compile_ar_by_rapp(self, cod_fiscale, cod_rapporto, cod_fondo):
        """
        http://93.57.3.196:8282/service.asmx?op=QST_MustCompileARByRap
        <cf>string</cf>
        <cod_rapp>long</cod_rapp>
        <cod_fon>string</cod_fon>
        """
        return self.client.QST_MustCompileARByRap(cod_fiscale, cod_rapporto, cod_fondo)

    def op_online_get_elenco_sottoscrizioni_aggiuntive(self, cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_GetElencoSottAgg
        <codCliente>int</codCliente>
        <codRapp>int</codRapp>
        <coddep>string</coddep>
        <CodFon>string</CodFon>
        <CodTC>string</CodTC>
        """
        return self.client.OpOnline_GetElencoSottAgg(cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc)[
            'OpOnline_GetElencoSottAggResult']  # fare ciclo for

    def op_online_check_can_operate(
            self, cod_fiscale, cod_cli, tipo_operazione, cod_rapp_from, cod_dep_from, cod_fon_from, tipo_contratto_from,
            cod_rapp_to, cod_dep_to, cod_fon_to, tipo_contratto_to, tipo_qta_movimento, valore_qta_movimento,
            num_rate_versate,
            check_manleva, iban
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_CheckCanOperate
        <cf>string</cf>
        <c_sot>long</c_sot>
        <tipoOperazione>SOTTOSCRIZIONE or SWITCH or RIMBORSO</tipoOperazione>
        <codRappFrom>string</codRappFrom>
        <codDepFrom>string</codDepFrom>
        <codFonFrom>string</codFonFrom>
        <tcFrom>string</tcFrom>
        <codRappTo>string</codRappTo>
        <codDepTo>string</codDepTo>
        <codFonTo>string</codFonTo>
        <tcTo>string</tcTo>
        <TipoQtaMovimento>QUOTE or IMPORTO or TOTALE</TipoQtaMovimento>
        <ValoreQtaMovimento>decimal</ValoreQtaMovimento>
        <NumRateVersate>long</NumRateVersate>
        <CheckManLeva>string</CheckManLeva>
        <IBAN>string</IBAN>
        """
        return self.client.OpOnline_CheckCanOperate(
            cod_fiscale, cod_cli, tipo_operazione, cod_rapp_from, cod_dep_from, cod_fon_from, tipo_contratto_from,
            cod_rapp_to, cod_dep_to, cod_fon_to, tipo_contratto_to, tipo_qta_movimento, valore_qta_movimento,
            num_rate_versate,
            check_manleva, iban
        )

    def check_cliente(self, cod_cliente, cod_fiscale, data_nascita):
        """
        Controlla se un cliente esistente è attivo.
        """
        data = self.client.CheckCliente(cod_cliente, cod_fiscale, format_data(data_nascita))
        return data['CheckClienteResult']

    def get_riepilogo(self, prodotto, partecipazione, cod_fiscale):
        """
        Torna tre valori aggregati per un certo prodotto / tipo di partecipazione di un cliente,
        dato dal suo codice fiscale
        """
        prodotto = prodotto.upper()
        partecipazione = partecipazione.upper()
        riepilogo = self.client.GetRiepilogo(prodotto, partecipazione, cod_fiscale)['GetRiepilogoResult']
        output = {}
        locale.setlocale(locale.LC_ALL, 'it_IT.utf8')
        output['valore_patrimonio'] = locale.format("%.2f", riepilogo['ValATT'], grouping=True, monetary=True)
        output['saldo_sottoscrizioni'] = locale.format("%.2f", riepilogo['SaldoSR'], grouping=True, monetary=True)
        output['utile_perdita'] = locale.format("%.2f", riepilogo['ProfitLoss'], grouping=True, monetary=True)
        return output

    def get_dettaglio(self, prodotto, partecipazione, cod_fiscale):
        """
        Torna una struttura dati col dettaglio per un certo prodotto / tipo di partecipazione di un cliente,
        dato dal suo codice fiscale
        """

        prodotto = prodotto.upper()
        partecipazione = partecipazione.upper()
        dettaglio = self.client.GetSintesiRapporti(prodotto, partecipazione, cod_fiscale)['GetSintesiRapportiResult']
        output = []
        for array_sr in dettaglio:
            sr = array_sr['SintesiRapporti']
            linea = dict()
            height = 30
            linea['valori_patrimoniali'] = locale.format("%.2f", sr['ValATT'], grouping=True, monetary=True)
            linea['tipo_partecipazione'] = sr['TipoPartecipazione']
            linea['descrizione_tipo_partecipazione'] = sr['TipoPartecipazioneDescr']
            linea['tipo_prodotto'] = sr['TipoProdotto']
            linea['descrizione_tipo_prodotto'] = sr['TipoProdottoDescr']
            linea['cod_prodotto'] = sr['CodProdotto']
            linea['descrizione_prodotto'] = sr['DescrProdotto']
            linea['saldo'] = locale.format("%.2f", sr['SaldoSR'], grouping=True, monetary=True)
            linea['contratto'] = sr['Contratto']
            linea['utili'] = locale.format("%.2f", sr['ProfitLoss'], grouping=True, monetary=True)
            linea['app'] = sr['App']
            linea['rapporto'] = sr['Rapporto']

            output.append(linea)
        return output

